@@ -3,11 +3,7 @@ package Promise::ES6;
 use strict;
 use warnings;
 
-our $VERSION = '0.15_01';
-
-use constant _has_current_sub => $^V ge v5.16.0;
-
-use if _has_current_sub(), feature => 'current_sub';
+our $VERSION = '0.15_02';
 
 use constant {
 
@@ -161,8 +157,22 @@ process’s global destruction, a warning is triggered.
 
 =item * If your application needs recursive promises (e.g., to poll
 iteratively for completion of a task), the C<current_sub> feature (i.e.,
-C<__SUB__>) may help you avoid memory leaks. (See this module’s source code
-for a substitute that works with pre-5.16 perls.)
+C<__SUB__>) may help you avoid memory leaks. In Perl versions that don’t
+support this feature you can imitate it thus:
+
+    use constant _has_current_sub => $^V ge v5.16.0;
+
+    use if _has_current_sub(), feature => 'current_sub';
+
+    my $cb;
+    $cb = sub {
+        my $current_sub = do {
+            no strict 'subs';
+            _has_current_sub() ? __SUB__ : eval '$cb';
+        };
+    }
+
+Of course, it’s better if you can avoid doing that. :)
 
 =item * Garbage collection before Perl 5.18 seems to have been buggy.
 If you work with such versions and end up chasing leaks,
@@ -272,54 +282,34 @@ sub new {
     return $self;
 }
 
+sub _propagate_if_needed_repromise {
+    my ($value_sr, $children_ar, $repromise_value_sr) = @_;
+
+    if ( _is_promise($$repromise_value_sr) ) {
+        $$repromise_value_sr->then(
+            sub { _propagate_if_needed_repromise( $value_sr, $children_ar, bless \do {my $v = $_[0]}, _RESOLUTION_CLASS ) },
+            sub { _propagate_if_needed_repromise( $value_sr, $children_ar, bless \do {my $v = $_[0]}, _REJECTION_CLASS ) },
+        );
+    }
+    else {
+        $$value_sr = $$repromise_value_sr;
+        bless $value_sr, ref($repromise_value_sr);
+
+        # It may not be necessary to empty out @$children_ar, but
+        # let’s do so anyway so Perl will delete references ASAP.
+        # It’s safe to do so because from here on $value_sr is
+        # no longer a pending value.
+        $_->_finish($value_sr) for splice @$children_ar;
+    }
+
+    return;
+}
+
 sub _propagate_if_needed {
     my ($value_sr, $children_ar) = @_;
 
-    # Avoid creating the closure if we can:
     if (@$children_ar || _is_promise($$value_sr)) {
-
-        my $cb;
-        $cb = sub {
-            my ($repromise_value_sr) = @_;
-
-            if ( _is_promise($$repromise_value_sr) ) {
-
-                # Accommodate Perl versions whose $@ handling is buggy
-                # by forgoing local():
-                my $old_err = $@;
-
-                my $current_sub = do {
-                    no strict 'subs';
-
-                    # The eval here mimics the “current_sub” feature:
-                    # a reference to the current subroutine
-                    # without actually closing on that reference.
-                    # This helps to prevent memory leaks.
-                    _has_current_sub() ? __SUB__ : eval '$cb';
-                };
-
-                $@ = $old_err;
-
-                my $in_reprom = $$repromise_value_sr->then(
-                    sub { $current_sub->( bless \do {my $v = $_[0]}, _RESOLUTION_CLASS ) },
-                    sub { $current_sub->( bless \do {my $v = $_[0]}, _REJECTION_CLASS ) },
-                );
-            }
-            else {
-                $$value_sr = $$repromise_value_sr;
-                bless $value_sr, ref($repromise_value_sr);
-
-                # It may not be necessary to empty out @$children_ar, but
-                # let’s do so anyway so Perl will delete references ASAP.
-                # It’s safe to do so because from here on $value_sr is
-                # no longer a pending value.
-                for my $subpromise (splice @$children_ar) {
-                    $subpromise->_finish($value_sr);
-                }
-            }
-        };
-
-        $cb->($value_sr);
+        _propagate_if_needed_repromise( $value_sr, $children_ar, $value_sr );
     }
 
     return;
